/* eslint-disable no-new-func */
/**
 * markdown document to nuxt.js vue page
 */
const path = require('path')

const glob = require('glob')
const fs = require('fs-extra')
const md2vue = require('md2vue')
const chokidar = require('chokidar')
const yamlFront = require('yaml-front-matter')

/**
 * build once
 */
exports.build = function ({ globs, output, vueTools }) {
  console.log(`[md2vue] release-build starts.`)

  const ret = globs
    .reduce((acc, pattern) => acc.concat(glob.sync(pattern)), [])
    .forEach(file => compile2vue(file, output, vueTools))

  console.log(`[md2vue] release-build done.`)

  return ret
}

/**
 * watch mode
 */
exports.watchBuild = function ({ globs, output, vueTools }) {
  console.log(`[md2vue] watch-build starts.`)

  return chokidar.watch(globs, {
    persistent: true
  })
  .on('add', file => {
    console.log(`[md2vue] ${file} added.`)
    compile2vue(file, output, vueTools)
  })
  .on('change', file => {
    console.log(`[md2vue] ${file} has been changed.`)
    compile2vue(file, output, vueTools)
  })
}

/**
 * compile markdown document to .vue code
 */
function compile2vue (file, output, vueTools) {
  // src/components/button/index.md => button => button.vue
  const name = path.basename(path.dirname(file))
  const defaults = {
    title: 'Component',
    layout: 'component',
    scrollTop: true,
    route: `component/${name}`
  }

  return fs.readFile(file)
    .then(buf => getFrontMatter(buf.toString()))
    .then(({ source, config }) => {
      const { layout, scrollTop, title, route } = Object.assign(defaults, config)
      const documentInfo = {
        layout: layout,
        scrollTop: scrollTop,
        head: new Function(`return { title: "${title}" }`)
      }

      let id = 0
      const customMarkups = function () {
        const type = typeof vueTools
        const uid = `demo-${route}-${id++}`

        if (type === 'string') {
          return vueTools
        }

        if (type === 'function') {
          return vueTools(name, uid)
        }

        return ''
      }

      const dest = path.join(output, `${route}.vue`)

      return fs.ensureFile(dest)
        .then(() =>
          md2vue(source, {
            customMarkups,
            documentInfo
          })
        )
        .then(code => fs.writeFile(dest, code))
        .catch(e => console.log(e))
    })
}

/**
 * 获取 markdown 文件的 yaml 信息
 */
function getFrontMatter (source) {
  const result = yamlFront.loadFront(source, '__mdContent')
  const { __mdContent } = result
  delete result.__mdContent

  return {
    source: __mdContent,
    config: result
  }
}
